/*

  __ MidnightSQL.com Server Evaluation __
  
Author:  Jennifer McCown, Jen@MidnightDBA.com
Version: 1.0
Source:  http://www.MidnightSQL.com

Purpose: This script gathers information about your server for a general, sight-unseen evaluation. 

 		 Feel free to use these results yourself, or get a __FREE server eval__ from me! Email the results 
		 to Jen@MidnightDBA.com with the subject "Free Server Eval".

		 Optional: For additional information, download Brent Ozar's sp_Blitz(TM) script from 
		 http://www.brentozar.com/blitz and install to the [master] database before you run this script.
		 Note that we're not affiliated with Brent Ozar LLC in any way; they just do good SQL.

IMPORTANT: This script comes with no guarantees or warranties! Though I've tried to ensure
		   the script is low impact and non-invasive, I can't guarantee how it will interact
		   with your system.
		   *** Test this script before you run it in a production environment, and use at your own risk.  ***

License: This work by Jennifer McCown is licensed under a Creative Commons Attribution 3.0 Unported License.
		 This means you can share, remix, and make commercial use of the script, so long as you attribute the 
		 original source as Jen McCown, www.MidnightSQL.com.
		   
*/
USE master;
go

/********************************************************************
-- Setup: drop and create table
     DROP TABLE #MyTable;
********************************************************************/

CREATE TABLE #MyTable
    (
      ID INT IDENTITY(1, 1) ,
      Name VARCHAR(500) ,
      Data VARCHAR(5000)
    );


INSERT  INTO #MyTable
        ( Name ,
          Data
        )
VALUES  ( '----- MASTER SECTION ------' ,
          'General Evaluation'
        );


/********************************************************************
						I. GENERAL EVALUATION
-- SERVER INFORMATION
********************************************************************/
INSERT  INTO #MyTable
        ( Name ,
          Data
        )
VALUES  ( '----- SECTION ------' ,
          'SERVER INFORMATION'
        );

CREATE TABLE #tmp
    (
      Id INT ,
      Name VARCHAR(800) ,
      InternalValue VARCHAR(800) ,
      CharacterValue VARCHAR(800)
    ) 

INSERT  INTO #tmp
        EXEC xp_msver;

UPDATE  #tmp
SET     [Name] = [Name] + ' (Mb)'
WHERE   Name = 'PhysicalMemory';
UPDATE  #tmp
SET     [Name] = [Name] + ': ';

WITH    CTE
--          AS ( SELECT   CAST(CAST (sqlserver_start_time AS DATE) AS VARCHAR(50)) AS sqlserver_start_date --*-- 2008+
--                        , '--UNKNOWN--' as  virtual_machine_type_desc
--               FROM     sys.dm_os_sys_info
--             )
-- 2005 version:
	  AS ( SELECT   -- CAST(CAST (sqlserver_start_time AS DATE) AS VARCHAR(50)) 
					NULL AS sqlserver_start_date --*-- 2008+
                        , '--UNKNOWN--' as  virtual_machine_type_desc
               FROM     sys.dm_os_sys_info
             )


    INSERT  INTO #tmp
            SELECT  -1 AS ID ,
                    'Server Name: ' AS [name] ,
                    NULL AS InternalValue ,
                    @@servername AS CharacterValue
            UNION
            SELECT  0 AS ID ,
                    'Eval Date: ' AS [name] ,
                    NULL AS InternalValue ,
                    CAST(GETDATE() AS VARCHAR(500)) AS CharacterValue
            UNION
            SELECT  100 AS [id] ,
                    'SQL Server start: ' AS [name] ,
                    NULL AS InternalValue ,
                    sqlserver_start_date AS CharacterValue
            FROM    CTE
            UNION
            SELECT  101 AS [id] ,
                    'Virtual Machine Type: ' AS [name] ,
                    NULL AS InternalValue ,
                    virtual_machine_type_desc AS CharacterValue
            FROM    CTE
            UNION
            SELECT  2 AS id ,
                    'ProductVersion [readable]: ' AS [name] ,
                    NULL AS InternalValue ,
                    @@version AS CharacterValue;

INSERT  INTO #Mytable
        SELECT  CAST([name] AS VARCHAR(500)) AS Name ,
                CAST(CharacterValue AS VARCHAR(5000)) AS Data
        FROM    #tmp
        WHERE   id NOT IN ( 5, 6, 9, 10, 11, 12, 13, 14, 17, 18, 20 )
        ORDER BY id ,
                [name];

DROP TABLE #tmp;
GO


/********************************************************************
						I. GENERAL EVALUATION
-- Databases: Size and location
*********************************************************************/

INSERT  INTO #MyTable
        ( Name ,
          Data
        )
VALUES  ( '----- SECTION ------' ,
          'DB FILE INFORMATION'
        );

SELECT  DB_NAME(database_id) AS Database_Name ,
        database_id ,
        file_id ,
        type ,
        type_desc ,
        name ,
        physical_name ,
        state ,
        state_desc ,
        size ,
        size * 8 / 1024 AS SizeInMb ,
        max_size ,
        growth ,
        growth * 8 / 1024 AS GrowthInMb , -- growth should really (generally) be by 1 Gb (1024 Mb)...with some exceptions.
        is_media_read_only ,
        is_read_only ,
        is_sparse ,
        is_percent_growth ,
        CASE WHEN is_percent_growth = 1 THEN 1
             WHEN type IN ( 0, 1 )
                  AND growth % 65536 <> 0 THEN 1
             ELSE 0
        END AS Has_Stupid_Growth_Rate
INTO    #tmp
FROM    sys.master_files;



INSERT  INTO #MyTable
        ( Name ,
          Data
        )
        SELECT  'DB File Count: ' AS Title ,
                COUNT(*) AS Data
        FROM    #tmp
        UNION
        SELECT  'DB Count: ' AS Title ,
                COUNT(DISTINCT database_id) AS Data
        FROM    #tmp
        UNION
        SELECT  'DB files not ONLINE: ' AS Title ,
                COUNT(*) AS Data
        FROM    #tmp
        WHERE   state <> 0;

INSERT  INTO #MyTable
        ( Name ,
          Data
        )
        SELECT TOP 1
                'MAX DB ROWS size (MB): ' AS Title ,
                SUM(size) * 8 / 1024 AS Data
        FROM    #tmp
        WHERE   type_desc = 'ROWS'
        GROUP BY database_id
        ORDER BY Data DESC

INSERT  INTO #MyTable
        ( Name ,
          Data
        )
        SELECT TOP 1
                'MIN DB ROWS size (MB): ' AS Title ,
                SUM(size) * 8 / 1024 AS Data
        FROM    #tmp
        WHERE   type_desc = 'ROWS'
        GROUP BY database_id
        ORDER BY Data ASC

INSERT  INTO #MyTable
        ( Name ,
          Data
        )
        SELECT  'AVG DB ROWS size (MB): ' AS Title ,
                ( SUM(size) / COUNT(*) ) * 8 / 1024 AS Data
        FROM    #tmp
        WHERE   type_desc = 'ROWS' 

INSERT  INTO #MyTable
        ( Name ,
          Data
        )
        SELECT  'File Types in Use: ' AS Title ,
                STUFF(( SELECT DISTINCT
                                ',' + type_desc
                        FROM    #tmp
                      FOR
                        XML PATH('')
                      ), 1, 1, '') AS Data;


INSERT  INTO #Mytable
        SELECT  'Count: Incorrect Growth Rate: ' AS Title ,
                SUM(Has_Stupid_Growth_Rate) AS data
        FROM    #tmp;


INSERT  INTO #Mytable
        SELECT  'Count: Limited max growth rate: ' AS Title ,
                COUNT(*) AS data
        FROM    #tmp
        WHERE   MAX_size NOT IN ( -1, 268435456 );
		

INSERT  INTO #Mytable
        SELECT DISTINCT
                'Drive letters in use: ' AS Title ,
                STUFF(( SELECT DISTINCT
                                ',' + LEFT(physical_name, 3)
                        FROM    #tmp
                      FOR
                        XML PATH('')
                      ), 1, 1, '') AS Data;

DROP TABLE #tmp;
GO

/********************************************************************
						I. GENERAL EVALUATION
-- Log File Info
*********************************************************************/

INSERT  INTO #MyTable
        ( Name ,
          Data
        )
VALUES  ( '----- SECTION ------' ,
          'LOG FILE INFORMATION'
        );


CREATE TABLE #tmp
    (
      [Database Name] SYSNAME ,
      logSizeMb FLOAT ,
      logSpaceUsed FLOAT ,
      STATUS INT
    );

INSERT  INTO #tmp
        EXEC ( 'DBCC SQLPERF (LOGSPACE)'
            );


INSERT  INTO #MyTable
        SELECT  'Total Log Files (Mb): ' ,
                SUM(LogSizeMb)
        FROM    #tmp

INSERT  INTO #MyTable
        SELECT TOP 1
                'Max Log Space Unused (Mb): ' ,
                [Database Name] + ', '
                + CAST (( LogSizeMb - ( logsizemb * logspaceused / 100 ) ) AS VARCHAR(100)) AS LogSpaceUnUsed
        FROM    #tmp
        GROUP BY [Database Name] ,
                ( LogSizeMb - ( logsizemb * logspaceused / 100 ) )
        ORDER BY ( LogSizeMb - ( logsizemb * logspaceused / 100 ) ) DESC

INSERT  INTO #MyTable
        SELECT  'Total Log Space Unused (Mb): ' ,
                SUM(LogSizeMb) - SUM(logsizemb * logspaceused / 100) AS LogSpaceUnUsed
        FROM    #tmp;

DROP TABLE #tmp;
GO


/********************************************************************
						I. GENERAL EVALUATION
-- Optional: sp_Blitz(TM) Result Summary 
*********************************************************************/
INSERT  INTO #MyTable
        ( Name ,
          Data
		)
VALUES  ( '----- SECTION ------' ,
          'SP_BLITZ(TM) RESULTS SUMMARY'
		);

IF EXISTS ( SELECT  name
            FROM    master.sys.objects
            WHERE   name = 'sp_blitz' ) 
    BEGIN
        CREATE TABLE #tmp
            (
              PRIORITY INT ,
              findingsgroup VARCHAR(100) ,
              finding VARCHAR(500) ,
              url VARCHAR(1000) ,
              details VARCHAR(5000) ,
              checkid INT
            );

        INSERT  INTO #tmp
                EXEC master..sp_Blitz;

        INSERT  INTO #MyTable
                SELECT  FindingsGroup + ': ' ,
                        Finding + ' (' + CAST(COUNT(*) AS VARCHAR(10)) + ')'
                FROM    #tmp
                WHERE   FindingsGroup NOT LIKE '%Thanks!%'
                        AND Finding NOT IN ( 'Foreign Keys Not Trusted',
                                             'Indexes Disabled',
                                             'Database Collation Mismatch' )
                GROUP BY Priority ,
                        FindingsGroup ,
                        Finding
                ORDER BY PRIORITY;

        DROP TABLE #tmp;
    END
ELSE 
    INSERT  INTO #MyTable
            ( Name ,
              Data
		    )
            SELECT  'sp_Blitz(TM) does not exist.' ,
                    '***For more info, download Brent Ozar''s sp_Blitz(TM) at http://www.brentozar.com/blitz, install it in master, and run this script again.'

GO

/********************************************************************
						I. GENERAL EVALUATION
-- Errors from Error log
*********************************************************************

INSERT  INTO #MyTable
        ( Name ,
          Data
        )
VALUES  ( '----- SECTION ------' ,
          'ERRORS FROM ERROR LOG - TBD'
        );
*/


/********************************************************************
						I. GENERAL EVALUATION
-- SP_Configure
*********************************************************************/

INSERT  INTO #MyTable
        ( Name ,
          Data
        )
VALUES  ( '----- SECTION ------' ,
          'SP_CONFIGURE'
        );


	EXEC SP_CONFIGURE 'show advanced options', 1;
	GO
	RECONFIGURE WITH OVERRIDE;
	GO
	CREATE TABLE #spConfigureOutput
		(
		  [name] VARCHAR(255) ,
		  [minimum] INT ,
		  [maximum] INT ,
		  [config_value] INT ,
		  [run_value] INT
		);
	 
	INSERT  INTO #spConfigureOutput
			EXEC SP_CONFIGURE

	INSERT  INTO #MyTable
			SELECT  name ,
					run_value
			FROM    #spConfigureOutput
			WHERE   name IN ( 'cost threshold for parallelism',
							  'max degree of parallelism',
							  'max server memory (MB)',
							  'optimize for ad hoc workloads' );
	GO
	EXEC SP_CONFIGURE 'show advanced options', 0;
	GO
	RECONFIGURE WITH OVERRIDE;
	GO

	DROP TABLE #spConfigureOutput;
	GO

/********************************************************************
						I. GENERAL EVALUATION
-- Traces 
*********************************************************************/

INSERT  INTO #MyTable
        ( Name ,
          Data
        )
VALUES  ( '----- SECTION ------' ,
          'TRACES'
        );
		
INSERT  INTO #mytable
        SELECT  'Trace id ' + CAST(id AS VARCHAR(10)) ,
                'Status = ' + CAST(status AS VARCHAR(10)) + ', Path = ' + path
        FROM    sys.traces;


/********************************************************************
						I. GENERAL EVALUATION
-- Trace Flags
--*********************************************************************/

INSERT  INTO #MyTable
        ( Name ,
          Data
        )
VALUES  ( '----- SECTION ------' ,
          'TRACE Flags'
        );
		
CREATE TABLE #tmp
    (
      [TraceFlag] INT ,
      Status TINYINT ,
      GLOBAL BIT ,
      SESSION TINYINT
    );

INSERT  INTO #tmp
        EXEC ( 'DBCC TRACESTATUS(-1);'
            );

INSERT  INTO #MyTable
        SELECT  'Trace flag ' + CAST(TraceFlag AS VARCHAR(10)) ,
                'Status=' + CAST(STATUS AS VARCHAR(2)) + ' GLOBAL='
                + CAST(GLOBAL AS VARCHAR(2)) + ' SESSION='
                + CAST(SESSION AS VARCHAR(10))
        FROM    #tmp

DROP TABLE #tmp;
GO




INSERT  INTO #MyTable
        ( Name ,
          Data
        )
VALUES  ( '----- MASTER SECTION ------' ,
          'Operational Evaluation'
        );

/********************************************************************
						II. OPERATIONAL EVALUATION
-- Backups
--*********************************************************************/

---------------------------------------------------------------
-- 1. Databases Missing a Data (aka Full) Back-Up Within Past 24 Hours 
--Src: http://www.mssqltips.com/sqlservertip/1601/script-to-retrieve-sql-server-database-backup-history-and-no-backups/
------------------------------------------------------------------------------------------- 
INSERT  INTO #MyTable
        ( Name ,
          Data
        )
VALUES  ( '----- SECTION ------' ,
          'MISSING BACKUPS'
        );
		
INSERT  INTO #MyTable
        SELECT  'Missing full backup: ' + msdb.dbo.backupset.database_name ,
        --MAX(msdb.dbo.backupset.backup_finish_date) AS last_db_backup_date ,
                DATEDIFF(hh, MAX(msdb.dbo.backupset.backup_finish_date),
                         GETDATE()) AS [Backup Age (Hours)]
        FROM    msdb.dbo.backupset
        WHERE   msdb.dbo.backupset.type = 'D'
        GROUP BY msdb.dbo.backupset.database_name
        HAVING  ( MAX(msdb.dbo.backupset.backup_finish_date) < DATEADD(hh, -24,
                                                              GETDATE()) )
        UNION  

--Databases without any backup history 
        SELECT  'Missing full backup: ' + master.dbo.sysdatabases.NAME AS database_name ,
        --NULL AS [Last Data Backup Date] ,
                9999 AS [Backup Age (Hours)]
        FROM    master.dbo.sysdatabases
                LEFT JOIN msdb.dbo.backupset ON master.dbo.sysdatabases.name = msdb.dbo.backupset.database_name
        WHERE   msdb.dbo.backupset.database_name IS NULL
                AND master.dbo.sysdatabases.name <> 'tempdb'
        ORDER BY 1



---------------------------------------------------------------
-- 1a. FULL mode databases Missing log Back-Up Within Past 24 Hours 
------------------------------------------------------------------------------------------- 
INSERT  INTO #MyTable
        SELECT  'Missing log backup: ' + msdb.dbo.backupset.database_name ,
                DATEDIFF(hh, MAX(msdb.dbo.backupset.backup_finish_date),
                         GETDATE()) AS [Backup Age (Hours)]
        FROM    msdb.dbo.backupset
        WHERE   msdb.dbo.backupset.type = 'L'
                AND msdb.dbo.backupset.recovery_model = 'FULL'
        GROUP BY msdb.dbo.backupset.database_name
        HAVING  ( MAX(msdb.dbo.backupset.backup_finish_date) < DATEADD(hh, -24,
                                                              GETDATE()) )
        UNION  

--Databases without any backup history 
        SELECT  'Missing full backup: ' + master.dbo.sysdatabases.NAME AS database_name ,
                9999 AS [Backup Age (Hours)]
        FROM    master.dbo.sysdatabases
                LEFT JOIN msdb.dbo.backupset ON master.dbo.sysdatabases.name = msdb.dbo.backupset.database_name
        WHERE   msdb.dbo.backupset.database_name IS NULL
                AND master.dbo.sysdatabases.name <> 'tempdb'
        ORDER BY 1;



/********************************************************************
						II. OPERATIONAL EVALUATION
-- Backup locations
--*********************************************************************/

INSERT  INTO #MyTable
        ( Name ,
          Data
        )
VALUES  ( '----- SECTION ------' ,
          'BACKUP LOCATIONS'
        );

WITH    CTE
          AS ( SELECT   CONVERT(VARCHAR(255), SERVERPROPERTY('Servername')) AS Server ,
                        msdb.dbo.backupset.database_name ,
                        MAX(msdb.dbo.backupset.backup_finish_date) AS last_db_backup_date
               FROM     msdb.dbo.backupmediafamily
                        INNER JOIN msdb.dbo.backupset ON msdb.dbo.backupmediafamily.media_set_id = msdb.dbo.backupset.media_set_id
               WHERE    msdb..backupset.type = 'D'
               GROUP BY msdb.dbo.backupset.database_name
             )
    INSERT  INTO #MyTable
            SELECT  DISTINCT
                    A.[Server] + ' backups on: ' ,
                    LEFT(B.physical_device_name, 3)
            FROM    CTE A
                    LEFT JOIN msdb.dbo.backupset bu ON bu.database_name = A.[database_name]
                                                       AND A.[last_db_backup_date] = BU.[backup_finish_date]
                    INNER JOIN msdb.dbo.backupmediafamily B ON B.media_set_id = BU.media_set_id
            WHERE   BU.type = 'D'
            ORDER BY 1 ,
                    2;


	
/********************************************************************
						II. OPERATIONAL EVALUATION
-- Job failures
--*********************************************************************/

INSERT  INTO #MyTable
        ( Name ,
          Data
        )
VALUES  ( '----- SECTION ------' ,
          'FAILING JOBS (Last 30 days)'
        );

INSERT  INTO #MyTable
        SELECT  'FAILED JOB: ' + j.name AS JobName ,
                'Error num: ' + CAST(jh.sql_message_id AS VARCHAR(20))
                + '; Last ran ' + CAST(jh.run_date AS VARCHAR(20)) + ' '
                + CAST(jh.run_time AS VARCHAR(20)) + '; Operator emailed? '
                + CASE jh.operator_id_emailed
                    WHEN 0 THEN 'Yes'
                    ELSE 'NO'
                  END
        FROM    msdb..sysjobhistory jh
                INNER JOIN msdb..sysjobs j ON jh.job_id = j.job_id
        WHERE   run_status = 0			-- failed
                AND run_date >= CAST(( CONVERT(VARCHAR(100), GETDATE() - 30, 112) ) AS INT)
        ORDER BY run_date DESC ,
                run_time DESC;
		


INSERT  INTO #MyTable
        ( Name ,
          Data
        )
VALUES  ( '----- MASTER SECTION ------' ,
          'Performance Evaluation'
        );

/********************************************************************
						III. PERFORMANCE EVALUATION
-- Disk offset and block size
*********************************************************************/
INSERT  INTO #MyTable
        ( Name ,
          Data
        )
VALUES  ( '----- SECTION ------' ,
          'Disk Partitioning'
        );
DECLARE @servername SYSNAME ,
    @cmd VARCHAR(8000);
SELECT  @servername = @@servername;
SELECT  @cmd = 'powershell.exe "Get-WmiObject Win32_DiskPartition -ComputerName '''
        + @servername
        + ''' | select SystemName, Name, Index, BlockSize, StartingOffset | Format-Table -AutoSize ';

IF ( SELECT value_in_use
     FROM   sys.configurations
     WHERE  name LIKE 'xp_cmdshell'
   ) = 1 
    BEGIN
        CREATE TABLE #tmp
            (
              id INT IDENTITY(1, 1) ,
              [OUTPUT] VARCHAR(8000)
            );
        INSERT  INTO #tmp
                EXEC xp_cmdshell @cmd;

        INSERT  INTO #MyTable
                SELECT  'DiskPartition:' ,
                        [output]
                FROM    #tmp
                WHERE   [output] IS NOT NULL
                        AND [output] NOT LIKE '-------%'
                ORDER BY id;

        DROP TABLE #tmp;
    END
ELSE 
    INSERT  INTO #MyTable
            SELECT  'Command line command to check disk partition: ' ,
                    @cmd;
go



/********************************************************************
						III. PERFORMANCE EVALUATION
-- Disk RAID Information - TDB
*********************************************************************

INSERT  INTO #MyTable
        ( Name ,
          Data
        )
VALUES  ( '----- SECTION ------' ,
          'Disk RAID Information - Ask your nework and SAN admins'
        );

********************************************************************
						III. PERFORMANCE EVALUATION
-- Disk Defrag Information - TDB
*********************************************************************

INSERT  INTO #MyTable
        ( Name ,
          Data
        )
VALUES  ( '----- SECTION ------' ,
          'Disk Defrag Information - High intensity operation; run separately'
        );
		*/

/********************************************************************
						III. PERFORMANCE EVALUATION
-- Waits and Queues 
*********************************************************************/

INSERT  INTO #MyTable
        ( Name ,
          Data
        )
VALUES  ( '----- SECTION ------' ,
          'TOP WAITS AND QUEUES'
        );

-- Source: http://technet.microsoft.com/en-us/magazine/hh781189.aspx
WITH    Waits
          AS ( SELECT   wait_type ,
                        wait_time_ms / 1000. AS wait_time_s ,
                        100. * wait_time_ms / SUM(wait_time_ms) OVER ( ) AS pct ,
                        ROW_NUMBER() OVER ( ORDER BY wait_time_ms DESC ) AS rn
               FROM     sys.dm_os_wait_stats
               WHERE    wait_type NOT IN ( 'CLR_SEMAPHORE', 'LAZYWRITER_SLEEP',
                                           'RESOURCE_QUEUE', 'SLEEP_TASK',
                                           'SLEEP_SYSTEMTASK',
                                           'SQLTRACE_BUFFER_FLUSH', 'WAITFOR',
                                           'LOGMGR_QUEUE', 'CHECKPOINT_QUEUE',
                                           'REQUEST_FOR_DEADLOCK_SEARCH',
                                           'XE_TIMER_EVENT', 'BROKER_TO_FLUSH',
                                           'BROKER_TASK_STOP',
                                           'CLR_MANUAL_EVENT',
                                           'CLR_AUTO_EVENT',
                                           'DISPATCHER_QUEUE_SEMAPHORE',
                                           'FT_IFTS_SCHEDULER_IDLE_WAIT',
                                           'XE_DISPATCHER_WAIT',
                                           'XE_DISPATCHER_JOIN' )
             )
    INSERT  INTO #mytable
            ( Name ,
              Data 
            )
            SELECT  W1.wait_type ,
                    CAST(W1.pct AS DECIMAL(12, 2)) AS pct
            FROM    Waits AS W1
                    INNER JOIN Waits AS W2 ON W2.rn <= W1.rn
            GROUP BY W1.rn ,
                    W1.wait_type ,
                    W1.wait_time_s ,
                    W1.pct
            HAVING  SUM(W2.pct) - W1.pct < 95;
 -- percentage threshold  


--*-- 
INSERT  INTO #MyTable
        ( Name ,
          Data
        )
VALUES  ( '----- MASTER SECTION ------' ,
          'THANKS!'
        );
INSERT  INTO #MyTable
        ( Name ,
          Data
		)
        SELECT  'Get a free server evaluation from MidnightSQL!', 'For your free SQL Server evaluation, email these results to Jen@MidnightDBA.com. 
	In the meantime, check out MidnightSQL.com and MidnightDBA.com for information on consulting and free IT tutorials!'

--*-- 
SELECT  *
FROM    #mytable
ORDER BY ID;

